/*

A jQuery edit in place plugin

Version 2.3.0

Authors:
	Dave Hauenstein
	Martin Häcker <spamfaenger [at] gmx [dot] de>

Project home:
	http://code.google.com/p/jquery-in-place-editor/

Patches with tests welcomed! For guidance see the tests  </spec/unit/>. To submit, attach them to the bug tracker.

License:
This source file is subject to the BSD license bundled with this package.
Available online: {@link http://www.opensource.org/licenses/bsd-license.php}
If you did not receive a copy of the license, and are unable to obtain it,
learn to use a search engine.

*/

(function($){
var tipo;

$.fn.editInPlace = function(options) {

	var settings = $.extend({}, $.fn.editInPlace.defaults, options);
	assertMandatorySettingsArePresent(settings);
	preloadImage(settings.saving_image);

	return this.each(function() {
		var dom = $(this);
		// This won't work with live queries as there is no specific element to attach this
		// one way to deal with this could be to store a reference to self and then compare that in click?
		if (dom.data('editInPlace'))
			return; // already an editor here
		dom.data('editInPlace', true);

		new InlineEditor(settings, dom).init();
	});
};

/// Switch these through the dictionary argument to $(aSelector).editInPlace(overideOptions)
/// Required Options: Either url or callback, so the editor knows what to do with the edited values.
$.fn.editInPlace.defaults = {
	url:				"", // string: POST URL to send edited content
	bg_over:			"#ffc", // string: background color of hover of unactivated editor
	bg_out:				"", // string: background color on restore from hover
	hover_class:		"",  // string: class added to root element during hover. Will override bg_over and bg_out
	show_buttons:		false, // boolean: will show the buttons: cancel or save; will automatically cancel out the onBlur functionality
	save_button:		'<button class="inplace_save">Save</button>', // string: image button tag to use as “Save” button
	cancel_button:		'<button class="inplace_cancel">Cancel</button>', // string: image button tag to use as “Cancel” button
	params:				"", // string: example: first_name=dave&last_name=hauenstein extra paramters sent via the post request to the server
	field_type:			"text", // string: "text", "textarea", or "select";  The type of form field that will appear on instantiation
	default_text:		"(Click here to add text)", // string: text to show up if the element that has this functionality is empty
	use_html:			false, // boolean, set to true if the editor should use jQuery.fn.html() to extract the value to show from the dom node
	textarea_rows:		10, // integer: set rows attribute of textarea, if field_type is set to textarea. Use CSS if possible though
	textarea_cols:		25, // integer: set cols attribute of textarea, if field_type is set to textarea. Use CSS if possible though
	select_text:		"Choose new value", // string: default text to show up in select box
	select_options:		"", // string or array: Used if field_type is set to 'select'. Can be comma delimited list of options 'textandValue,text:value', Array of options ['textAndValue', 'text:value'] or array of arrays ['textAndValue', ['text', 'value']]. The last form is especially usefull if your labels or values contain colons)
	text_size:			null, // integer: set cols attribute of text input, if field_type is set to text. Use CSS if possible though

	// Specifying callback_skip_dom_reset will disable all saving_* options
	saving_text:		undefined, // string: text to be used when server is saving information. Example "Saving..."
	saving_image:		"", // string: uses saving text specify an image location instead of text while server is saving
	saving_animation_color: 'transparent', // hex color string, will be the color the pulsing animation during the save pulses to. Note: Only works if jquery-ui is loaded

	value_required:		false, // boolean: if set to true, the element will not be saved unless a value is entered
	element_id:			"element_id", // string: name of parameter holding the id or the editable
	update_value:		"update_value", // string: name of parameter holding the updated/edited value
	original_value:		'original_value', // string: name of parameter holding the updated/edited value
	original_html:		"original_html", // string: name of parameter holding original_html value of the editable /* DEPRECATED in 2.2.0 */ use original_value instead.
	save_if_nothing_changed:	false,  // boolean: submit to function or server even if the user did not change anything
	on_blur:			"save", // string: "save" or null; what to do on blur; will be overridden if show_buttons is true
	cancel:				"", // string: if not empty, a jquery selector for elements that will not cause the editor to open even though they are clicked. E.g. if you have extra buttons inside editable fields

	// All callbacks will have this set to the DOM node of the editor that triggered the callback

	callback:			null, // function: function to be called when editing is complete; cancels ajax submission to the url param. Prototype: function(idOfEditor, enteredText, orinalHTMLContent, settingsParams, callbacks). The function needs to return the value that should be shown in the dom. Returning undefined means cancel and will restore the dom and trigger an error. callbacks is a dictionary with two functions didStartSaving and didEndSaving() that you can use to tell the inline editor that it should start and stop any saving animations it has configured. /* DEPRECATED in 2.1.0 */ Parameter idOfEditor, use $(this).attr('id') instead
	callback_skip_dom_reset: false, // boolean: set this to true if the callback should handle replacing the editor with the new value to show
	success:			null, // function: this function gets called if server responds with a success. Prototype: function(newEditorContentString)
	error:				null, // function: this function gets called if server responds with an error. Prototype: function(request)
	error_sink:			function(idOfEditor, errorString) { alert(errorString); }, // function: gets id of the editor and the error. Make sure the editor has an id, or it will just be undefined. If set to null, no error will be reported. /* DEPRECATED in 2.1.0 */ Parameter idOfEditor, use $(this).attr('id') instead
	preinit:			null, // function: this function gets called after a click on an editable element but before the editor opens. If you return false, the inline editor will not open. Prototype: function(currentDomNode). DEPRECATED in 2.2.0 use delegate shouldOpenEditInPlace call instead
	postclose:			null, // function: this function gets called after the inline editor has closed and all values are updated. Prototype: function(currentDomNode). DEPRECATED in 2.2.0 use delegate didCloseEditInPlace call instead
	delegate:			null // object: if it has methods with the name of the callbacks documented below in delegateExample these will be called. This means that you just need to impelment the callbacks you are interested in.
};

// Lifecycle events that the delegate can implement
// this will always be fixed to the delegate
var delegateExample = {
	// called while opening the editor.
	// return false to prevent editor from opening
	shouldOpenEditInPlace: function(aDOMNode, aSettingsDict, triggeringEvent) {},
	// return content to show in inplace editor
	willOpenEditInPlace: function(aDOMNode, aSettingsDict) {},
	didOpenEditInPlace: function(aDOMNode, aSettingsDict) {},

	// called while closing the editor
	// return false to prevent the editor from closing
	shouldCloseEditInPlace: function(aDOMNode, aSettingsDict, triggeringEvent) {},
	// return value will be shown during saving
	willCloseEditInPlace: function(aDOMNode, aSettingsDict) {},
	didCloseEditInPlace: function(aDOMNode, aSettingsDict) {},

	missingCommaErrorPreventer:''
};


function InlineEditor(settings, dom) {
	this.settings = settings;
	this.dom = dom;
	this.originalValue = null;
	this.didInsertDefaultText = false;
	this.shouldDelayReinit = false;
};

$.extend(InlineEditor.prototype, {

	init: function() {
		this.setDefaultTextIfNeccessary();
		this.connectOpeningEvents();
	},

	reinit: function() {
		if (this.shouldDelayReinit)
			return;

		this.triggerCallback(this.settings.postclose, /* DEPRECATED in 2.1.0 */ this.dom);
		this.triggerDelegateCall('didCloseEditInPlace');

		this.markEditorAsInactive();
		this.connectOpeningEvents();
	},

	setDefaultTextIfNeccessary: function() {
		if('' !== this.dom.html())
			return;

		this.dom.html(this.settings.default_text);
		this.didInsertDefaultText = true;
	},

	connectOpeningEvents: function() {
		var that = this;
		this.dom
			.bind('mouseenter.editInPlace', function(){ that.addHoverEffect(); })
			.bind('mouseleave.editInPlace', function(){ that.removeHoverEffect(); })
			.bind('openEditor.editInPlace', function(anEvent){
				if (tablonPapelera() && !esPopup()) return;
				that.openEditor(anEvent);
				that.dblclick = false;
			})
			.bind('dblclick.editInPlace', function(anEvent){
								that.dblclick = true;
                // Se guarda posición del doble click para hacer scroll al punto
                try {
                    var numPost = parseInt(anEvent.currentTarget.id.substr(5));
                    if (!esPopup()) {
                        that.scrollEditor = anEvent.pageY - $('#postit' + numPost).position().top + $('#texto' + numPost).scrollTop() - 70;
                    } else {
                        that.scrollEditor = anEvent.pageY + $('#postit' + numPost).scrollTop() - 80;
                    }
                    if (that.scrollEditor < 200) that.scrollEditor = 0;
                } catch (e) {that.scrollEditor = 0}

                that.openEditor(anEvent);
                if (that.element.className.indexOf('classTransparent') != -1) {
                    $('.classTransparent').each(function(){
                        $(this).find('.pencilImg').click();
                    });
                    $('.classTransparent').removeClass('classTransparent');
                }
        });
	},

	disconnectOpeningEvents: function() {
	 	// prevent re-opening the editor when it is already open
		this.dom.unbind('.editInPlace');
	},

	addHoverEffect: function() {
		if (this.settings.hover_class)
			this.dom.addClass(this.settings.hover_class);
		else
			this.dom.css("background-color", this.settings.bg_over);
	},

	removeHoverEffect: function() {
		if (this.settings.hover_class)
			this.dom.removeClass(this.settings.hover_class);
		else
			this.dom.css("background-color", this.settings.bg_out);
	},

	openEditor: function(anEvent) {
		if (tablonPapelera() && !esPopup()) return;

		// if (!esPopup() && pckry) {
		// 	pckry.destroy();
		// }

		if ( ! this.shouldOpenEditor(anEvent))
			return;

		var element = anEvent.target;
		this.element = element;

		if (element.className != 'tabLabel' && element.className.indexOf('tabPaneles2') == -1 && element.className.indexOf('tabA')== -1)
		{

			if (esPopup())	//nota en popup
			{
				this.medidasPopup = getPopupData(winId);
			}

			while (element != document && element.className.indexOf('postit') == -1)
				element = element.parentNode;

			//this.tipo = parseInt($.trim(element.className.substring(6,element.className.indexOf('ui-draggable'))));
			this.numPost = parseInt(element.id.substr(6));
			this.tipo = localStorage["tipoA"+this.numPost];
			this.sinMarco = localStorage["sinMarco"+this.numPost];
			this.element = element;
			if (!esPopup())
			{
				var edWidth = parseInt(localStorage["width"+this.numPost]);
				if (edWidth > 260) 	$(this.element).width(edWidth + 90 + 'px');
				else $(this.element).width('400px');
				$(this.element).css('min-width','400px');
//                if (postitNoVisibleEntero(this.numPost)) {
//                    console.log('habría que agrandarlo');
//                    var posicionUltimo = $('#texto' + this.numPost).children().last().position().top + 250;
//                    if (posicionUltimo > $('#postit'+this.numPost).height()) {
//                        $('#postit'+this.numPost).height(posicionUltimo);
//                    }
//                }
			}
			else $(this.element).css('min-width','400px');
            if (!esPopup() || webkitBrowser()) $(this.element).height('auto');
			$(this.element).css('z-index','1000');
			$('#texto'+this.numPost).html(localStorage["postit"+this.numPost]);
			document.getElementById('postit'+this.numPost).style.transform = '';
		}

		this.settings.save_button = '<div class="inplace_save" title="CTRL+S">' +  chrome.i18n.getMessage("aceptar") +'</div>';
		this.settings.cancel_button = '<div class="inplace_cancel" title="CTRL+Q">' +  chrome.i18n.getMessage("cancelar") +'</div>';

		this.disconnectOpeningEvents();
		this.removeHoverEffect();
		this.removeInsertedDefaultTextIfNeccessary();
		this.saveOriginalValue();
		this.markEditorAsActive();

		if (element.className.indexOf('tabLabel') == -1 && element.className.indexOf('tabPaneles2') == -1 && element.className.indexOf('tabA')== -1)
		{
			var num = this.numPost;
			var selector = '<div class="selector" id="selector' + this.numPost + '">' +
								'<div id="idamarillo' + num + '" class="idamarillo2"></div>'+
								'<div id="idverde' + num + '" class="idverde2"></div>'+
								'<div id="idlila'+  num + '" class="idlila2"></div>'+
								'<div id="idnaranja' + num + '" class="idnaranja2"></div>'+
								'<div id="idrosa' + num + '" class="idrosa2"></div>'+
								'<div id="idrojo' + num + '" class="idrojo2"></div>'+
								'<div id="idazul' + num + '" class="idazul2"></div>' +
                                '<div id="notepad' + num + '" class="notepadBut"></div>' +
								'<div id="customColor' + num + '" class="customColor2"><div></div></div>';
				//if (!esPopup()) selector +=	'<img class="pinImgSelecEdit" src="imagenes/pushpins/pushpin11.png"/>';
				 selector +=	'<img class="pinImgSelecEdit" src="imagenes/pushpins/pushpin11.png"/>';
				selector +=		'<div class="sinMarcoDiv"><label class="sinMarcoLabel">Sin marco</label><input id="sinMarco' + num  + '" type="checkbox" name="Sinc"></div>' +
							'</div>';

			//if (!esPopup()) {
				var pinSelector = '<div class="selectorPins">';
				if (localStorage['pushpin' + num])
					pinSelector += '<img class="pinImgSelec" src="imagenes/pushpins/pushpin' + localStorage['pushpin' + num] + '.png"/>';
				else
					pinSelector += '<img class="pinImgSelec" src="imagenes/pushpins/pushpin11.png"/>';

				pinSelector += '<img class="pinImgSelec" src="imagenes/pushpins/pushpin11.png"/>' +
						'<img class="pinImgSelec" src="imagenes/pushpins/pushpin21.png"/>' +
						'<img class="pinImgSelec" src="imagenes/pushpins/pushpin31.png"/>' +
					'</div>';

				$('#postit'+num + ' .bordesup').after(pinSelector);
			//}

			$('#postit'+num).find('.pinImgSelec').on('click',function(evt) {clickImgSelec(evt,num)});

			//$('#postit'+this.numPost +'>div:nth-child(1)').after(selector);
			if (esPopup() && localStorage["sinMarco"+num] == 'true')
			{
				//setTimeout(function() {$('.parteSuperior').removeClass('parteSuperior2')},20);
				$('.parteSuperior').removeClass('parteSuperior2')
				$('#parteSuperior'+this.numPost).after(selector);
				$('.parteSuperior').show();
				//$('#selector'+this.numPost).css('margin-top','65px');
				$('#selector'+this.numPost).css('margin-top','10px');
			}
			else
            {
                if (localStorage["sinMarco"+num] == 'true')
                {
                    $('#bordesup' + num).height(20);
                    $('#parteSuperior' + num).height(30);
                    $('#fondoIconos' + num).hide();
                    $('#parteSuperior'+this.numPost).after(selector)
                }
                else $('#borr'+this.numPost).after(selector);
            }
            $('.pinImgSelecEdit').attr('title',chrome.i18n.getMessage("selecPin"));
            $('.pinImgSelec:first-child').attr('title',chrome.i18n.getMessage("quitaPin"));
			$('.sinMarcoLabel').text(chrome.i18n.getMessage("sinMarco"));
			$('#idamarillo'+this.numPost).on("click",function(){cambioPostit('1','postit' + num)});
			$('#idverde'+this.numPost).on("click",function(){cambioPostit('2','postit' + num)});
			$('#idlila'+this.numPost).on("click",function(){cambioPostit('3','postit' + num)});
			$('#idnaranja'+this.numPost).on("click",function(){cambioPostit('4','postit' + num)});
			$('#idrosa'+this.numPost).on("click",function(){cambioPostit('5','postit' + num)});
			$('#idrojo'+this.numPost).on("click",function(){cambioPostit('6','postit' + num)});
			$('#idazul'+this.numPost).on("click",function(){cambioPostit('7','postit' + num)});
            $('#notepad'+this.numPost).on("click",function(){cambioPostit('80','postit' + num)});
			$('#customColor'+this.numPost).css('background-color','#' + localStorage["tipoA"+ num]);

            iniZoom(this.numPost);

			$('.customColor2').ColorPicker({
				color: '#' + localStorage["tipoA"+ num],
				onSubmit: function(hsb, hex, rgb, el) {
					$(el).val(hex);
					$(el).ColorPickerHide();
				},
				onBeforeShow: function (hsb, hex, rgb, el) {
					$(this).ColorPickerSetColor('#' + localStorage["tipoA"+ num]);
				},
				onChange: function (hsb, hex, rgb, el) {
					$('#customColor2').css('background-color','#' + hex);
					cambioPostit(hex,'postit' + num);
					// if (getContrastYIQ(hex) == 'black')
					// {
					// 	$('#postit' + num).css('color','white');
					// }


				},
			})
			.bind('keyup', function(){
				$(this).ColorPickerSetColor(this.value);
			});

		$("#sinMarco"+num).click(function() {
			localStorage["sinMarco"+num] = $( "#sinMarco"+num).is(':checked');
			$('#postit'+num).toggleClass('postitSinmarco');
			$('#texto'+num).height('auto');
		});

		if (localStorage["sinMarco"+num] == 'true')
		{
			$('#parteSuperior'+num).show();
			$('#sinMarco'+num).attr('checked', true);
//			if (!esPopup())
//			{
//				$('#parteSuperior'+num).slideDown(50)
//										.height(55);
//				$('#postit'+num).animate({top: "-=55"},50);
//			}
		}
		else $('#sinMarco'+num).attr('checked', false);

			// Se pone height auto siempre ?????
			$('#texto'+num).height('auto');

			$('#pencilImg'+this.numPost).hide();
			$('#postit' + num).attr('data-tipo',this.tipo);
//			if ($('#texto' + num).height() > 1600)
//			{
//			 	$(this.element).css('overflow','auto');
//			}
			ponAlFrentePost(this.numPost);
		}
		this.replaceContentWithEditor();

		if (esPopup() && !elementInViewport(document.querySelector('.inplace_save')))	//nota en popup
		{
			var that = this;
			if (localStorage["sinMarco"+this.numPost] == 'true')
			{
				setTimeout(function(){
                   chrome.runtime.sendMessage({resizeWindow: {
                       winId: winId,
                       width : $('#postit'+that.numPost).outerWidth()+10,
                       height: ($('.inplace_save').position().top+100 < screen.height ? $('.inplace_save').position().top+100: screen.height - 100 )
                   }});
				},600);
				setTimeout(function(){
                   chrome.runtime.sendMessage({resizeWindow: {
                       winId: winId,
                       width : $('#postit'+that.numPost).outerWidth()+10,
                       height: ($('.inplace_save').position().top+100 < screen.height ? $('.inplace_save').position().top+100: screen.height - 100 )
                   }});
				},1000);
				setTimeout(function(){
					if ($('.inplace_save').length > 0)
                       chrome.runtime.sendMessage({resizeWindow: {
                           winId: winId,
                           width : $('#postit'+that.numPost).outerWidth()+10,
                           height: ($('.inplace_save').position().top+100 < screen.height ? $('.inplace_save').position().top+100: screen.height - 100 )
                       }});
				},2000);
			}
			else
			{
				setTimeout(function(){
                   chrome.runtime.sendMessage({resizeWindow: {
                       winId: winId,
                       width : $('#postit'+that.numPost).outerWidth()+30,
                       height: ($('.inplace_save').position().top+100 < screen.height ? $('.inplace_save').position().top+100: screen.height - 100 )
                   }});
				},600);
				setTimeout(function(){
                   chrome.runtime.sendMessage({resizeWindow: {
                       winId: winId,
                       width : $('#postit'+that.numPost).outerWidth()+50,
                       height: ($('.inplace_save').position().top+100 < screen.height ? $('.inplace_save').position().top+100: screen.height - 100 )
                   }});
				},1000);
				setTimeout(function(){
					if ($('.inplace_save').length > 0) {
                       chrome.runtime.sendMessage({resizeWindow: {
                           winId: winId,
                           width : $('#postit'+that.numPost).outerWidth()+50,
                           height: ($('.inplace_save').position().top+100 < screen.height ? $('.inplace_save').position().top+100: screen.height - 100 )
                       }});
                   }
				},2000);
			}
			// setTimeout(function(){
			// 	window.resizeTo($('#postit'+that.numPost).width()+50,$('.inplace_save').position().top+100);
			// },8000);

		}
		// this.setInitialValue();
		// this.workAroundMissingBlurBug();
		// this.connectClosingEventsToEditor();
		// this.triggerDelegateCall('didOpenEditInPlace');
	},

	shouldOpenEditor: function(anEvent) {
		if (this.isClickedObjectCancelled(anEvent.target))
			return false;

		if (false === this.triggerCallback(this.settings.preinit, /* DEPRECATED in 2.1.0 */ this.dom))
			return false;

		if (false === this.triggerDelegateCall('shouldOpenEditInPlace', true, anEvent))
			return false;

		return true;
	},

	removeInsertedDefaultTextIfNeccessary: function() {
		if ( ! this.didInsertDefaultText
			|| this.dom.html() !== this.settings.default_text)
			return;

		this.dom.html('');
		this.didInsertDefaultText = false;
	},

	isClickedObjectCancelled: function(eventTarget) {
		if ( ! this.settings.cancel)
			return false;

		var eventTargetAndParents = $(eventTarget).parents().andSelf();
		var elementsMatchingCancelSelector = eventTargetAndParents.filter(this.settings.cancel);
		return 0 !== elementsMatchingCancelSelector.length;
	},

	saveOriginalValue: function() {
		if (this.settings.use_html)
		{
			this.originalValue = this.dom.html();
			//this.originalValue = this.originalValue.replace(/<br>/g,'\n');
		}
		else
			this.originalValue = trim(this.dom.text());
	},

	restoreOriginalValue: function() {
		this.setClosedEditorContent(this.originalValue);
	},

	setClosedEditorContent: function(aValue) {
		if (this.settings.use_html)
			this.dom.html(aValue);
		else
			this.dom.text(aValue);
	},

	workAroundMissingBlurBug: function() {
		// Strangely, all browser will forget to send a blur event to an input element
		// when another one is created and selected programmatically. (at least under some circumstances).
		// This means that if another inline editor is opened, existing inline editors will _not_ close
		// if they are configured to submit when blurred.

		// Using parents() instead document as base to workaround the fact that in the unittests
		// the editor is not a child of window.document but of a document fragment
		var ourInput = this.dom.find(':input');
		this.dom.parents(':last').find('.editInPlace-active :input').not(ourInput).blur();
	},

	replaceContentWithEditor: function() {
		var buttons_html  = (this.settings.show_buttons) ? this.settings.save_button + ' ' + this.settings.cancel_button : '';
		var editorElement = this.createEditorElement(); // needs to happen before anything is replaced
		/* insert the new in place form after the element they click, then empty out the original element */
		this.dom.html('<form class="inplace_form" style="display: inline; margin: 0; padding: 0;"></form>')
			.find('form')
				.append(editorElement)
				.append(buttons_html);
		$('.inplace_field').css('background','transparent');
		if (this.element.className.indexOf('tabLabel') == -1 && this.element.className.indexOf('tabPaneles2') == -1 && this.element.className.indexOf('tabA')== -1)
		{
			//if (!tinyMCE.get('textAreaEdit' + this.numPost)) tinyMCE.get('textAreaEdit' + this.numPost).remove();
			crearEditor(this.numPost, function(){
				this.setInitialValue();
			}.bind(this));
			var that = this;
			if (!esPopup()) {
				setTimeout(function(){
					$('html, body').animate({ scrollTop: $("#postit" + that.numPost).offset().top - 150, scrollLeft: $("#postit" + that.numPost).offset().left - 100 }, 200, function () { if (!that.dblclick) muestraAyuda("ayudaEditarNota"); });
				},400)
			}
            setTimeout(function(){
                if (that.scrollEditor > 0) {
                    $('#postit' + that.numPost +' iframe').contents().find('html').animate({scrollTop:that.scrollEditor},300);
                }
            },400)

			this.continuaEditor();
		}
		else this.continuaEditor();

	},
	cambiaReabre: function(){ console.log('cambia');localStorage['reabriendoEditor'] = 'false';},
	continuaEditor: function(){
		// this.setInitialValue();
		this.workAroundMissingBlurBug();
		this.connectClosingEventsToEditor();
		this.triggerDelegateCall('didOpenEditInPlace');
        //$('#postit'+this.numPost).height('auto');
	},

	createEditorElement: function() {
		if (-1 === $.inArray(this.settings.field_type, ['text', 'textarea', 'select']))
			throw "Unknown field_type <fnord>, supported are 'text', 'textarea' and 'select'";

		var editor = null;
		if ("select" === this.settings.field_type)
			editor = this.createSelectEditor();
		else if ("text" === this.settings.field_type)
			editor = $('<input type="text" ' + this.inputNameAndClass()
				+ ' size="' + this.settings.text_size  + '" />');
		else if ("textarea" === this.settings.field_type)
			editor = $('<textarea ' + this.inputNameAndClass()
				+ ' rows="' + this.settings.textarea_rows + '" '
				+ ' id="textAreaEdit' + this.numPost +'"'
				+ ' cols="' + this.settings.textarea_cols + '" />');

		return editor;
	},

	setInitialValue: function() {
		var initialValue = this.triggerDelegateCall('willOpenEditInPlace', this.originalValue);
		var editor = this.dom.find(':input');
		editor.val(initialValue);
		tipo = this.tipo;
		// Workaround for select fields which don't contain the original value.
		// Somehow the browsers don't like to select the instructional choice (disabled) in that case
		if (editor.val() !== initialValue)
			editor.val(''); // selects instructional choice

		if (this.element.className.indexOf('tabLabel') == -1 && this.element.className.indexOf('tabPaneles2') == -1 && this.element.className.indexOf('tabA')== -1)
		{
			if (localStorage["editando"+this.numPost] != "" && localStorage["editando"+this.numPost] != undefined &&  localStorage["editando"+this.numPost] != "undefined")
			{
				initialValue = localStorage["editando"+this.numPost];
				if (!esPopup())
				{
					$('html, body').animate({scrollTop:$(this.element).offset().top - 150},200);
          $('html, body').animate({scrollLeft:$(this.element).offset().left - 100}, 200);
				}
			}
			if (tinyMCE.get('textAreaEdit' + this.numPost) != undefined) {
				tinyMCE.get('textAreaEdit' + this.numPost).setContent(initialValue);
				tinyMCE.get('textAreaEdit' + this.numPost).focus();
				tinyMCE.get('textAreaEdit' + this.numPost).on('change', function (evt) {
					try {
						localStorage["editando" + evt.target.id.substr(12)] = tinyMCE.get('textAreaEdit' + evt.target.id.substr(12)).getContent();
					} catch (e) {
						console.log(e);
					}
				});
				cambiaAparenciaEditor(buscarTipo("postit" + this.numPost), "textAreaEdit" + this.numPost);
				var that = this;
				tinyMCE.get('textAreaEdit' + this.numPost).addShortcut('ctrl+q', '', function () {
					that.handleCancelEditor();
				});
			}
			else
			{
				var numPostEd = this.numPost;
				var that = this;
				intervalEd = setInterval (function(){
						if (tinyMCE.get('textAreaEdit' + numPostEd) != undefined)
						{
							tinyMCE.get('textAreaEdit' + numPostEd).setContent(initialValue);
							tinyMCE.get('textAreaEdit' + numPostEd).focus();
							tinyMCE.get('textAreaEdit' + numPostEd).on('change',function(evt) {
								localStorage["editando"+evt.target.id.substr(12)] = evt.level.content;
							});
							cambiaAparenciaEditor(buscarTipo("postit"+numPostEd),"textAreaEdit"+numPostEd);
							clearInterval(intervalEd);
							tinyMCE.get('textAreaEdit' + numPostEd).addShortcut('ctrl+q', '', function(){
								that.handleCancelEditor();
							});
						}
				}, 50);
			}
			//tinyMCE.get('textAreaEdit' + this.numPost).autoresize_min_height = 700;
			//tinyMCE.get('textAreaEdit' + this.numPost).execCommand('mceAutoResize');
		}
	},
	inputNameAndClass: function() {
		return ' name="inplace_value" class="inplace_field" ';
	},

	createSelectEditor: function() {
		var editor = $('<select' + this.inputNameAndClass() + '>'
			+	'<option disabled="true" value="">' + this.settings.select_text + '</option>'
			+ '</select>');

		var optionsArray = this.settings.select_options;
		if ( ! $.isArray(optionsArray))
			optionsArray = optionsArray.split(',');

		for (var i=0; i<optionsArray.length; i++) {
			var currentTextAndValue = optionsArray[i];
			if ( ! $.isArray(currentTextAndValue))
				currentTextAndValue = currentTextAndValue.split(':');

			var value = trim(currentTextAndValue[1] || currentTextAndValue[0]);
			var text = trim(currentTextAndValue[0]);

			var option = $('<option>').val(value).text(text);
			editor.append(option);
		}

		return editor;
	},

	connectClosingEventsToEditor: function() {
		var that = this;
		function cancelEditorAction(anEvent) {
			that.handleCancelEditor(anEvent);
			return false; // stop event bubbling
		}
		function saveEditorAction(anEvent) {
			that.handleSaveEditor(anEvent);
			return false; // stop event bubbling
		}

		var form = this.dom.find("form");

		form.find(".inplace_field").focus().select();
		form.find(".inplace_cancel").click(cancelEditorAction);
		form.find(".inplace_save").click(saveEditorAction);

		if ( ! this.settings.show_buttons) {
				// TODO: Firefox has a bug where blur is not reliably called when focus is lost
				//       (for example by another editor appearing)
			if ("save" === this.settings.on_blur)
				form.find(".inplace_field").blur(saveEditorAction);
			else
				form.find(".inplace_field").blur(cancelEditorAction);

			// workaround for msie & firefox bug where it won't submit on enter if no button is shown
			// if ($.browser.mozilla || $.browser.msie)
			// 	this.bindSubmitOnEnterInInput();
		}

		form.keyup(function(anEvent) {
			// allow canceling with escape
			var escape = 27;
			if (escape === anEvent.which)
				return cancelEditorAction();
		});

		// workaround for webkit nightlies where they won't submit at all on enter
		// REFACT: find a way to just target the nightlies
		// if ($.browser.safari)
		// 	this.bindSubmitOnEnterInInput();


		form.submit(saveEditorAction);
	},

	bindSubmitOnEnterInInput: function() {
		if ('textarea' === this.settings.field_type)
			return; // can't enter newlines otherwise

		var that = this;
		this.dom.find(':input').keyup(function(event) {
			var enter = 13;
			if (enter === event.which)
				return that.dom.find('form').submit();
		});

	},

	handleCancelEditor: function(anEvent) {
		// REFACT: remove duplication between save and cancel
		if (false === this.triggerDelegateCall('shouldCloseEditInPlace', true, anEvent))
			return;

		//var enteredText = this.dom.find(':input').val();
		if (this.element.className.indexOf('tabLabel') == -1 && this.element.className.indexOf('tabPaneles2') == -1 && this.element.className.indexOf('tabA')== -1)
		{
			var enteredText  = tinyMCE.get('textAreaEdit' + this.numPost).getContent();
			enteredText = this.triggerDelegateCall('willCloseEditInPlace', enteredText);
			if (tinyMCE.get('textAreaEdit' + this.numPost).editorContainer.className.indexOf('mce-fullscreen') > -1)
			{
				tinyMCE.get('textAreaEdit' + this.numPost).execCommand('mceFullScreen')
			}

			tinyMCE.get('textAreaEdit' + this.numPost).remove();

			cambioPostit(this.tipo,'postit' + this.numPost);
			this.restoreOriginalValue();

			if ($('#texto' + this.numPost).height() > 1600)
			{
			 	$(this.element).scrollTop(0);
			 	$(this.element).css('overflow','hidden');
			}
			if (localStorage["width"+this.numPost] == 'auto')
				$(this.element).width(localStorage["width"+this.numPost]);
			else
				$(this.element).width(localStorage["width"+this.numPost]+'px');

			$(this.element).css('min-width','170px');

			$(this.element).height('auto');
			if ($(this.element).height() + 30 < localStorage["height"+this.numPost])
				$(this.element).height(localStorage["height"+this.numPost]+'px');

			localStorage["editando"+this.numPost] = "";localStorage["editandoTipo"] = "";

			localStorage["sinMarco"+this.numPost] = this.sinMarco;
			$('#selector'+this.numPost).remove();
			if (!esPopup()) document.getElementById('postit'+this.numPost).style.transform = 'rotate(' + localStorage["rotacion"+this.numPost]+ 'deg)';

			$('#postit'+this.numPost+ ' .selectorPins').remove();
			$('#postit'+this.numPost+ ' .subSelectorPins').remove();

			if (localStorage["pushpin"+this.numPost]) {
				$("#postit"+this.numPost).find('.pinImg').remove();
			 	$("#postit"+this.numPost).append('<img class="pinImg" data-pushpin="' + localStorage["pushpin"+this.numPost] +'" src="imagenes/pushpins/pushpin' + localStorage["pushpin"+this.numPost] +'.png" />');
			}


			if (localStorage["sinMarco"+this.numPost] == 'true' && !esPopup())
			{
                $('#postit'+this.numPost).remove();
                muestraPostit(this.numPost);
               $("#texto"+this.numPost).height($("#texto"+this.numPost).height()-7)
			}
		}
		else var enteredText = this.dom.find(':input').val();

		// asterisco esto sin mucha idea
		/*if (hasContent(enteredText)
			&& ! this.isDisabledDefaultSelectChoice())
			this.setClosedEditorContent(enteredText);*/
		this.reinit();
        if (esPopup() && localStorage["sinMarco"+this.numPost] == 'true')
        {
            $('.parteSuperior').addClass('parteSuperior2');
        }

        if (esPopup())	//nota en popup
		{
			if (notaWeb()) {
				if (nuevaNota()) {
					localStorage.removeItem('postit' + this.numPost);
					localStorage.removeItem("tablon" + this.numPost);
					localStorage.removeItem("tipoA" + this.numPost);
					localStorage.removeItem("X" + this.numPost);
					localStorage.removeItem("Y" + this.numPost);
					localStorage.removeItem("width" + this.numPost);
					localStorage.removeItem("height" + this.numPost);
					localStorage.removeItem("treeId" + this.numPost);
					localStorage.removeItem("fecha" + this.numPost);
					localStorage.removeItem("updated" + this.numPost);
					localStorage.removeItem("calendar" + this.numPost);
					localStorage.removeItem("sinMarco" + this.numPost);
					localStorage.removeItem("userComp" + this.numPost);
					localStorage.removeItem("nickComp" + this.numPost);
					localStorage.removeItem("errorConexion" + this.numPost);
					localStorage.removeItem("rotacion" + this.numPost);
					localStorage.removeItem("pushpin" + this.numPost);
					localStorage.removeItem("notaWeb" + this.numPost);
				}
                chrome.runtime.sendMessage({cierraVentana: winId});
			}
			else {
                chrome.runtime.sendMessage({resizeWindow: {
                    winId: winId,
                    width : this.medidasPopup.width,
                    height: this.medidasPopup.height
                }});
				//window.resizeTo(this.medidasPopup.width,this.medidasPopup.height);
				window.moveTo(this.medidasPopup.X,this.medidasPopup.Y);
                $('#postit'+this.numPost).remove();
                if (!notaWeb()) muestraPostitPop(this.numPost);
				$('.postitPop').css('width','100%');
//				setTimeout(function() {$('.postitPop').height($(window).height()-10)},200);
			}
		}
        else if (localStorage["sinMarco"+this.numPost] == 'true')
        {
            $('#bordesup' + this.numPost).height(0);
            $('#parteSuperior' + this.numPost).height(0);
            $('#fondoIconos' + this.numPost).show();
            $('#fondoIconos' + this.numPost).css('position','absolute');
        }
        if (Prism) Prism.highlightAll();
        // if (!esPopup()) {
        // 	inicializaTablonMasonry();
        // }
	},

	handleSaveEditor: function(anEvent) {
		if (false === this.triggerDelegateCall('shouldCloseEditInPlace', true, anEvent))
			return;

		//var enteredText = this.dom.find(':input').val();
		if (this.element.className.indexOf('tabLabel') == -1 && this.element.className.indexOf('tabPaneles2') == -1 && this.element.className.indexOf('tabA')== -1)
		{
			//var enteredText  = CKEDITOR.instances.textAreaEdit.getData();
			//var enteredText  = tinyMCE.activeEditor.getContent();
			if ($('#texto' + this.numPost).height() > 1600)
			{
			 	$(this.element).scrollTop(0);
			 	$(this.element).css('overflow','hidden');
			}
	//		if (localStorage["editando"+this.numPost] == "" || localStorage["editando"+this.numPost] == undefined)
				var enteredText  = tinyMCE.get('textAreaEdit' + this.numPost).getContent();
			// else
			// 	var enteredText  = localStorage["editando"+this.numPost];

			localStorage["editando"+this.numPost] = "";//localStorage["editandoTipo"] = "";
			localStorage["tipoA"+this.numPost] = this.tipo;
			enteredText = this.triggerDelegateCall('willCloseEditInPlace', enteredText);

			// probar más
			// var scrollTop = $('#postit' + this.numPost +' iframe').contents().find('html').scrollTop() - 200;
		}
		else var enteredText = this.dom.find(':input').val();

	/*	if (this.isDisabledDefaultSelectChoice()
			|| this.isUnchangedInput(enteredText)) {
			this.handleCancelEditor(anEvent);
			return;
		}*/

		/*if (this.didForgetRequiredText(enteredText)) {
			this.handleCancelEditor(anEvent);
			this.reportError("Error: You must enter a value to save this field");
			return;
		}*/

		// this.showSaving(enteredText);
		try {
			tinyMCE.get('textAreaEdit' + this.numPost).remove();
		} catch(e) {
			console.log(e);
		}

		if (this.settings.callback)
			this.handleSubmitToCallback(enteredText);
		else
			this.handleSubmitToServer(enteredText);

		if (this.element.className.indexOf('tabLabel') == -1 && this.element.className.indexOf('tabPaneles2') == -1 && this.element.className.indexOf('tabA')== -1)
		{
			$(this.element).width(localStorage["width"+this.numPost]+'px');
			$(this.element).css('min-width','170px');
			//$(this.element).height(localStorage["height"+this.numPost]+'px');
			$(this.element).height('auto');
			if ($(this.element).height() + 30 < localStorage["height"+this.numPost])
				$(this.element).height(localStorage["height"+this.numPost]+'px');

			localStorage["height"+this.numPost] = $(this.element).height();
			// if (tinyMCE.getInstanceById('textAreaEdit' + this.numPost) != null)
			// {
  	// 			tinyMCE.execCommand('mceRemoveControl', true, 'textAreaEdit' + this.numPost);
			// }

			$('#selector'+this.numPost).remove();
			localStorage["editando"+this.numPost] = "";
		}
		this.markEditorAsInactive();

        if (esPopup() && localStorage["sinMarco"+this.numPost] == 'true')
        {
            $('.parteSuperior').addClass('parteSuperior2');
        //    $('.parteSuperior').show();
     //       $('#selector'+this.numPost).css('margin-top','10px');
        }
        else if (localStorage["sinMarco"+this.numPost] == 'true')
        {
            $('#bordesup' + this.numPost).height(0);
            $('#parteSuperior' + this.numPost).height(0);
            $('#fondoIconos' + this.numPost).show();
            $('#fondoIconos' + this.numPost).css('position','absolute');
        }

		if (esPopup())	//nota en popup
		{
			if (notaWeb()) {

		// var pushpin = $('#postit' + this.numPost + ' .pinImg').data('pushpin') || 0;
		// if (pushpin) localStorage["pushpin"+this.numPost] = pushpin;

				if (nuevaNota()) {
					var nota = {
						postit: enteredText,
						tipoA: localStorage['tipoA'+this.numPost],
						numPost: this.numPost,
						url: paramUrl(),
						tabId: paramTabId()
					}
					chrome.runtime.sendMessage({creaNotaNuevaWeb: nota});
				} else {
                    var datosNota = {
                        numPost : this.numPost
                    }
                    chrome.runtime.sendMessage({cambiaNota: datosNota});
                }
                chrome.runtime.sendMessage({cierraVentana: winId});
			}
			else {
                chrome.runtime.sendMessage({resizeWindow: {
                    winId: winId,
                    width : this.medidasPopup.width,
                    height: this.medidasPopup.height
                }});
				//window.resizeTo(this.medidasPopup.width,this.medidasPopup.height);
				window.moveTo(this.medidasPopup.X,this.medidasPopup.Y);
			}
		}
		// @TODO pendiente de refinar
		// if (scrollTop && scrollTop > 200) {
		// 	$('#postit' + this.numPost).animate({scrollTop: scrollTop}, 200);
		// }
        if (Prism) Prism.highlightAll();
        // if (!esPopup()) {
        // 	inicializaTablonMasonry();
        // }
	},

	didForgetRequiredText: function(enteredText) {
		return this.settings.value_required
			&& ("" === enteredText
				|| undefined === enteredText
				|| null === enteredText);
	},

	isDisabledDefaultSelectChoice: function() {
		return this.dom.find('option').eq(0).is(':selected:disabled');
	},

	isUnchangedInput: function(enteredText) {
		return ! this.settings.save_if_nothing_changed
			&& this.originalValue === enteredText;
	},

	showSaving: function(enteredText) {
		if (this.settings.callback && this.settings.callback_skip_dom_reset)
			return;

		var savingMessage = enteredText;
		if (hasContent(this.settings.saving_text))
			savingMessage = this.settings.saving_text;
		if(hasContent(this.settings.saving_image))
			// REFACT: alt should be the configured saving message
			savingMessage = $('<img />').attr('src', this.settings.saving_image).attr('alt', savingMessage);
		this.dom.html(savingMessage);
	},

	handleSubmitToCallback: function(enteredText) {
		// REFACT: consider to encode enteredText and originalHTML before giving it to the callback
		this.enableOrDisableAnimationCallbacks(true, false);
		var newHTML = this.triggerCallback(this.settings.callback, /* DEPRECATED in 2.1.0 */ this.id(), enteredText, this.originalValue,
			this.settings.params, this.savingAnimationCallbacks());

		if (this.settings.callback_skip_dom_reset)
			; // do nothing
		else if (undefined === newHTML) {
			// failure; put original back
			this.reportError("Error: Failed to save value: " + enteredText);
			this.restoreOriginalValue();
		}
		else
			// REFACT: use setClosedEditorContent
			this.dom.html(newHTML);

		if (this.didCallNoCallbacks()) {
			this.enableOrDisableAnimationCallbacks(false, false);
			this.reinit();
		}
	},

	handleSubmitToServer: function(enteredText) {
		var data = this.settings.update_value + '=' + encodeURIComponent(enteredText)
			+ '&' + this.settings.element_id + '=' + this.dom.attr("id")
			+ ((this.settings.params) ? '&' + this.settings.params : '')
			+ '&' + this.settings.original_html + '=' + encodeURIComponent(this.originalValue) /* DEPRECATED in 2.2.0 */
			+ '&' + this.settings.original_value + '=' + encodeURIComponent(this.originalValue);

		this.enableOrDisableAnimationCallbacks(true, false);
		this.didStartSaving();
		var that = this;
		$.ajax({
			url: that.settings.url,
			type: "POST",
			data: data,
			dataType: "html",
			complete: function(request){
				that.didEndSaving();
			},
			success: function(html){
				var new_text = html || that.settings.default_text;

				/* put the newly updated info into the original element */
				// FIXME: should be affected by the preferences switch
				that.dom.html(new_text);
				// REFACT: remove dom parameter, already in this, not documented, should be easy to remove
				// REFACT: callback should be able to override what gets put into the DOM
				that.triggerCallback(that.settings.success, html);
			},
			error: function(request) {
				that.dom.html(that.originalHTML); // REFACT: what about a restorePreEditingContent()
				if (that.settings.error)
					// REFACT: remove dom parameter, already in this, not documented, can remove without deprecation
					// REFACT: callback should be able to override what gets entered into the DOM
					that.triggerCallback(that.settings.error, request);
				else
					that.reportError("Failed to save value: " + request.responseText || 'Unspecified Error');
			}
		});
	},

	// Utilities .........................................................

	triggerCallback: function(aCallback /*, arguments */) {
		if ( ! aCallback)
			return; // callback wasn't specified after all

		var callbackArguments = Array.prototype.slice.call(arguments, 1);
		return aCallback.apply(this.dom[0], callbackArguments);
	},

	/// defaultReturnValue is only used if the delegate returns undefined
	triggerDelegateCall: function(aDelegateMethodName, defaultReturnValue, optionalEvent) {
		// REFACT: consider to trigger equivalent callbacks automatically via a mapping table?
		if ( ! this.settings.delegate
			|| ! $.isFunction(this.settings.delegate[aDelegateMethodName]))
			return defaultReturnValue;

		var delegateReturnValue =  this.settings.delegate[aDelegateMethodName](this.dom, this.settings, optionalEvent);
		return (undefined === delegateReturnValue)
			? defaultReturnValue
			: delegateReturnValue;
	},

	reportError: function(anErrorString) {
		this.triggerCallback(this.settings.error_sink, /* DEPRECATED in 2.1.0 */ this.id(), anErrorString);
	},

	// REFACT: this method should go, callbacks should get the dom node itself as an argument
	id: function() {
		return this.dom.attr('id');
	},

	markEditorAsActive: function() {
		this.dom.addClass('editInPlace-active');
	},

	markEditorAsInactive: function() {
		this.dom.removeClass('editInPlace-active');
	},

	// REFACT: consider rename, doesn't deal with animation directly
	savingAnimationCallbacks: function() {
		var that = this;
		return {
			didStartSaving: function() { that.didStartSaving(); },
			didEndSaving: function() { that.didEndSaving(); }
		};
	},

	enableOrDisableAnimationCallbacks: function(shouldEnableStart, shouldEnableEnd) {
		this.didStartSaving.enabled = shouldEnableStart;
		this.didEndSaving.enabled = shouldEnableEnd;
	},

	didCallNoCallbacks: function() {
		return this.didStartSaving.enabled && ! this.didEndSaving.enabled;
	},

	assertCanCall: function(methodName) {
		if ( ! this[methodName].enabled)
			throw new Error('Cannot call ' + methodName + ' now. See documentation for details.');
	},

	didStartSaving: function() {
		this.assertCanCall('didStartSaving');
		this.shouldDelayReinit = true;
		this.enableOrDisableAnimationCallbacks(false, true);

		this.startSavingAnimation();
	},

	didEndSaving: function() {
		this.assertCanCall('didEndSaving');
		this.shouldDelayReinit = false;
		this.enableOrDisableAnimationCallbacks(false, false);
		this.reinit();

		this.stopSavingAnimation();
	},

	startSavingAnimation: function() {
		var that = this;
		this.dom
			.animate({ backgroundColor: this.settings.saving_animation_color }, 400)
			.animate({ backgroundColor: 'transparent'}, 400, 'swing', function(){
				// In the tests animations are turned off - i.e they happen instantaneously.
				// Hence we need to prevent this from becomming an unbounded recursion.
				setTimeout(function(){ that.startSavingAnimation(); }, 10);
			});
	},

	stopSavingAnimation: function() {
		this.dom
			.stop(true)
			.css({backgroundColor: ''});
	},

	missingCommaErrorPreventer:''
});



// Private helpers .......................................................

function assertMandatorySettingsArePresent(options) {
	// one of these needs to be non falsy
	if (options.url || options.callback)
		return;

	throw new Error("Need to set either url: or callback: option for the inline editor to work.");
}

/* preload the loading icon if it is configured */
function preloadImage(anImageURL) {
	if ('' === anImageURL)
		return;

	var loading_image = new Image();
	loading_image.src = anImageURL;
}

function trim(aString) {
	return aString
		.replace(/^\s+/, '')
		.replace(/\s+$/, '');
}

function hasContent(something) {
	if (undefined === something || null === something)
		return false;

	if (0 === something.length)
		return false;

	return true;
}

})(jQuery);
